<?php

namespace Picea\Caching;

class Opcache implements Cache {

    protected string $cachePath = "";

    public function __construct(?string $cachePath = null)
    {
        $this->cachePath = $cachePath ?? sys_get_temp_dir();
    }

    public function handle(string $fileName) : bool
    {
        if ( file_exists($path = $this->filePath($fileName)) ) {
            require_once($path);
            return true;
        }

        return false;
    }

    public function saveFile(string $fileName, string $content) : bool
    {
        $this->validateCachePath();

        file_put_contents($this->filePath($fileName), $content);

        return false;
    }

    public function filePath(string $fileName) : string
    {
        return $this->cachePath . $fileName;
    }

    protected function validateCachePath() : bool
    {
        if ( ! file_exists($this->cachePath) ) {
            mkdir($this->cachePath, 0750, true);
        }

        if ( ! is_writeable($this->cachePath) ) {
            throw new \RuntimeException(
                sprintf("Given cache path `%s` is not writeable by `%s`", $this->cachePath, static::class)
            );

            return false;
        }

        return true;
    }
}
