<?php declare(strict_types=1);

namespace Picea;

use Closure;
use Picea\Language\LanguageRegistration;
use Psr\Http\Message\ResponseInterface;

class Picea implements LanguageRegistration
{
    const DEFAULT_BUILDER_TEMPLATE = "/Builder/ClassTemplate.php";

    public Compiler\Context $context;

    public Compiler $compiler;

    public LanguageRegistration $languageRegistration;

    public string $builderTemplatePath;

    public Closure $responseHtml;

    public Caching\Cache $cache;

    public function __construct(
        ?Closure $responseHtml = null,
        ?Cache $cache = null,
        ?Context $context = null,
        ?Compiler $compiler = null,
        ?LanguageRegistration $languageRegistration = null,
        ?string $builderTemplatePath = null
    ){
        $this->response = $responseHtml;
        $this->cache = $cache ?? new Caching\Memory("");
        $this->context = $context ?? new Compiler\BaseContext();
        $this->compiler = $compiler ?? $this->instanciateCompiler();
        $this->languageRegistration = $languageRegistration ?? new Language\DefaultRegistrations();
        $this->builderTemplatePath = $builderTemplatePath ?? dirname(__FILE__) . static::DEFAULT_BUILDER_TEMPLATE;
    }

    public function outputHtml(string $viewPath, array $variables) : ResponseInterface
    {
        if ( $this->response ?? false ) {
            if ( false === $content = $this->cache->handle($viewPath) ) {
                # if ( )
            }

            $source = $this->compileSource($source ?? "test");

            $response = $this->response;
            return $response("abc");
        }

        throw new \InvalidArgumentException("No \Psr\Http\Message\ResponseInterface closure provided. Please provide one using the constructor or assinging it to the class variable `responseHtml`.");
    }

    public function compileSource(string $source) : array
    {
        $this->compiler->loadSourceCode($source);

        return [
            'context' => $context = clone $this->context,
            'source'  => $this->compiler->compile($context),
        ];
    }

    public function compileFile(string $filePath) : array
    {
        if (! file_exists($filePath) ) {
            throw new \InvalidArgumentException("File `$filePath` cannot be found or there is a permission problem.");
        }

        if ( false === $fileContent = file_get_contents($filePath) ) {
            throw new \ErrorException("Given file could not be opened `$filePath`. This could indicate a permission misconfiguration on your file or folder.");
        }

        $this->compiler->loadSourceCode($fileContent);

        return [
            'context' => $context = clone $this->context,
            'source'  => $this->compiler->compile($context),
        ];
    }

    public function buildFromSource(string $source) : array
    {
        $tmpFolder = sys_get_temp_dir();
        $builder = $this->instanciateBuilder();
        $compiledSource = $this->compileSource($source);
        list($namespace, $className, $compiledSource) = $builder->build($compiledSource['context'], $compiledSource['source']) ;

        $path = "$tmpFolder/$className.php";
        file_put_contents($path, $compiledSource);

        return [
            'path' => $path,
            'namespace' => $namespace,
            'className' => $className,
        ];
    }

    public function buildFromPath(string $path) : string
    {

    }

    public function instanciateCompiler() : Compiler
    {
        return new Compiler($this->languageRegistration);
    }

    public function instanciateBuilder() : Builder
    {
        return new Builder($this->builderTemplatePath);
    }

    public function registerAll(Compiler $compiler) : void
    {
        $this->registerSyntax($compiler);
        $this->registerControlStructure($compiler);
        $this->registerExtension($compiler);
    }

    public function registerSyntax(Compiler $compiler) : void
    {
        $this->languageRegistration->registerSyntax($compiler);
    }

    public function registerControlStructure(Compiler $compiler) : void
    {
        $this->languageRegistration->registerControlStructure($compiler);
    }

    public function registerExtension(Compiler $compiler) : void
    {
        $this->languageRegistration->registerExtension($compiler);
    }
}
